@@ -45,7 +45,7 @@
 #include "parse_config.h"
 
 /* A/52 */
-static ao_instance_t * output;
+static ao_instance_t * output = NULL;
 static sample_t * samples;
 static int disable_dynrng = 0;
 static clocktime_t a52_decode_data(uint8_t *start, uint8_t *end);
@@ -58,13 +58,6 @@ static int attach_stream_buffer(uint8_t stream_id, uint8_t subtype, int shmid);
 
 static void handle_events(MsgEventQ_t *q, MsgEvent_t *ev);
 
-
-static char *program_name;
-
-#ifdef NEW_SYNC
-static FILE *outfile;
-#endif
-
 static int ctrl_data_shmid;
 static ctrl_data_t *ctrl_data;
 static ctrl_time_t *ctrl_time;
@@ -78,22 +71,122 @@ static char *data_buf_shmaddr;
 static int msgqid = -1;
 static MsgEventQ_t *msgq;
 
-static int flush_to_scrnr = -1;
+static int flush_to_scrid = -1;
 static int prev_scr_nr = 0;
 
+static int speaker_flags = -1;
 
+char *program_name;
 
 void usage()
 {
-  fprintf(stderr, "Usage: %s  [-m <msgid>]\n", 
-	  program_name);
+  fprintf(stderr, "Usage: %s  [-m <msgid>]\n", program_name);
 }
 
+static int get_speaker_flags(void)
+{
+  int front, rear, sub;
+  int result = 0;
+  
+  front = get_num_front_speakers();
+  rear = get_num_rear_speakers();
+  sub = get_num_sub_speakers();
+
+  if(front == 2 && rear == 0) {
+    result = A52_STEREO;
+    /* if(dolby) result = A52_DOLBY; */
+  } else if(front == 3 && rear == 0) {
+    result = A52_3F;
+  } else if(front == 2 && rear == 2) {
+    result = A52_2F2R;
+  } else if(front == 3 && rear == 2) {
+    result = A52_3F2R;
+  } else if(front == 2 && rear == 1) {
+    result = A52_2F1R;
+  } else if(front == 3 && rear == 1) {
+    result = A52_3F1R;
+  }
+  if(sub == 1) {
+    result |= A52_LFE;
+  }
+  return result;
+}
+
+
+static void open_output(int flags)
+{
+  int i;
+  int driver_num = 0;
+  ao_driver_t * drivers;
+  int ch_avail;
+
+  /* Ideally, we would just pass `flags' through to liba52, since that
+   * seems to be how it works internally, however until it offers
+   * such an interface, we figure out which "driver" it needs.
+   */
+
+  switch(flags & A52_CHANNEL_MASK) {
+  case A52_CHANNEL:
+  case A52_STEREO:
+  case A52_DOLBY:
+    ch_avail = 2;
+    break;
+  case A52_3F:
+  case A52_2F1R:
+    ch_avail = 3;
+    break;
+  case A52_3F1R:
+  case A52_2F2R:
+    ch_avail = 4;
+    break;
+  case A52_3F2R:
+    ch_avail = 5;
+    break;
+  default:
+    ch_avail = 0;
+  }
+
+  if(flags & A52_LFE) {
+    ch_avail++;
+  }
+
+  drivers = ao_drivers ();
+  for(i = 0; drivers[i].name; i++) {
+    if(!strcmp("oss", drivers[i].name)
+      || !strcmp("solaris", drivers[i].name)) {
+      if(ch_avail == 2) {
+        printf("opening a 2 channel audio driver\n");
+        driver_num = i;
+      }
+    } else if(!strcmp("oss4", drivers[i].name)) {
+      if(ch_avail == 4) {
+        printf("opening a 4 channel audio driver\n");
+        driver_num = i;
+      }
+    } else if(!strcmp("oss6", drivers[i].name)) {
+      if(ch_avail == 6) {
+        printf("opening a 6 channel audio driver\n");
+        driver_num = i;
+      }
+    }
+  }
+
+  if(output) {
+    ao_close(output);
+    free(output);
+  }
+  output = ao_open(drivers[driver_num].open, get_audio_device());
+  if(output == NULL) {
+    FATAL("Can not open audio output\n");
+    exit(1);
+  }
+}
 
 int main(int argc, char *argv[])
 {
   MsgEvent_t ev;
-  int c; 
+  int c;
+  
   program_name = argv[0];
   
   /* Parse command line options */
@@ -116,33 +209,28 @@ int main(int argc, char *argv[])
     }
   }
   
-  if(parse_config() < -1) {
-    fprintf(stderr, "FATAL[a52_decoder]: Couldn't read config files\n");
+  if(parse_config() == -2) {
+    FATAL("Couldn't read config files\n");
+    exit(1);
   }
 
   {
-    ao_driver_t * drivers;
     uint32_t accel;
     accel = MM_ACCEL_MLIB;
-    
-    drivers = ao_drivers();
-    output = ao_open(drivers[0].open, get_audio_device());
-    if(output == NULL) {
-        fprintf(stderr, "Can not open audio output\n");
-        exit(1);
-    }
+
+    open_output(get_speaker_flags());
 
     samples = a52_init(accel);
     if(samples == NULL) {
-        fprintf(stderr, "A52 init failed\n");
+        FATAL("A/52 init failed\n");
         exit(1);
     }
   }
 
   if(msgqid != -1) {
     if((msgq = MsgOpen(msgqid)) == NULL) {
-      fprintf(stderr, "a52: couldn't get message q\n");
-      exit(-1);
+      FATAL("couldn't get message q\n");
+      exit(1);
     }
     
     ev.type = MsgEventQRegister;
@@ -157,14 +245,13 @@ int main(int argc, char *argv[])
     }
 
   } else {
-    fprintf(stderr, "what?\n");
+    FATAL("what? need a msgid\n");
   }
 
   while(1) {
     get_q();
   }
 
-
   return 0;
 }
 
@@ -178,7 +265,7 @@ static void handle_events(MsgEventQ_t *q, MsgEvent_t *ev)
     break;
   case MsgEventQFlushData:
     DPRINTF(1, "a52: got flush\n");
-    flush_to_scrnr = ev->flushdata.to_scrnr;
+    flush_to_scrid = ev->flushdata.to_scrid;
     break;
   case MsgEventQDecodeStreamBuf:
     DPRINTF(1, "a52: got stream %x, %x buffer \n",
@@ -205,7 +292,7 @@ static void handle_events(MsgEventQ_t *q, MsgEvent_t *ev)
     }
     break;
   default:
-    fprintf(stderr, "a52: unrecognized event type: %d\n", ev->type);
+    DNOTE("unrecognized event type: %d\n", ev->type);
     break;
   }
 }
@@ -236,11 +323,11 @@ int attach_stream_buffer(uint8_t stream_id, uint8_t subtype, int shmid)
   char *shmaddr;
   q_head_t *q_head;
 
-  fprintf(stderr, "a52: shmid: %d\n", shmid);
+  //DNOTE("a52_decoder: shmid: %d\n", shmid);
   
   if(shmid >= 0) {
     if((shmaddr = shmat(shmid, NULL, SHM_SHARE_MMU)) == (void *)-1) {
-      perror("ac3: attach_decoder_buffer(), shmat()");
+      perror("ac52_decoder: attach_decoder_buffer(), shmat()");
       return -1;
     }
     
@@ -253,7 +340,7 @@ int attach_stream_buffer(uint8_t stream_id, uint8_t subtype, int shmid)
   
   if(shmid >= 0) {
     if((shmaddr = shmat(shmid, NULL, SHM_SHARE_MMU)) == (void *)-1) {
-      perror("ac3: attach_data_buffer(), shmat()");
+      perror("a52_decoder: attach_data_buffer(), shmat()");
       return -1;
     }
     
@@ -319,8 +406,8 @@ int get_q()
   off = data_elem->off;
   len = data_elem->len;
 
-  if(flush_to_scrnr != -1) {
-    if(flush_to_scrnr != scr_nr) {
+  if(flush_to_scrid != -1) {
+    if(ctrl_time[scr_nr].scr_id < flush_to_scrid) {
 
       q_head->read_nr = (q_head->read_nr+1)%q_head->nr_of_qelems;
       
@@ -332,13 +419,12 @@ int get_q()
 	q_head->writer_requests_notification = 0;
 	ev.type = MsgEventQNotify;
 	if(MsgSendEvent(msgq, q_head->writer, &ev, 0) == -1) {
-	  fprintf(stderr, "a52: couldn't send notification\n");
+	  WARNING("couldn't send notification\n");
 	}
       }
-
       return 0;
     } else {
-      flush_to_scrnr = -1;
+      flush_to_scrid = -1;
     }
   }
   if(ctrl_data->speed == 1.0) {
@@ -357,8 +443,6 @@ int get_q()
       if(ctrl_time[scr_nr].offset_valid == OFFSET_NOT_VALID) {
 	if(PTS_DTS_flags & 0x2) {
 	  
-
-	  
 	  // time_offset is our guess to how much is in the output q
 
 	  if(TIME_S(last_rt) != -1) {
@@ -381,7 +465,6 @@ int get_q()
 	    */
 	  }
 	  
-	  
 	  fprintf(stderr, "*rt: %ld.%09ld, last_rt: %ld.%09ld\n "
 		  "bt: %ld.%09ld,  tmptime: %ld.%09ld\n scr: %ld.%09ld\n",
 		  (long)TIME_S(real_time), (long)TIME_SS(real_time),
@@ -513,12 +596,12 @@ int get_q()
       if(TIME_SS(time_offset) < 0 || TIME_S(time_offset) < 0) {
 	TIME_S(time_offset) = 0;
 	TIME_SS(time_offset) = 0;
-	fprintf(stderr, "a52: resetting offset\n");
+	NOTE("resetting offset\n");
 	set_sync_point(&ctrl_time[scr_nr],
 		       &real_time,
 		       &scr_time,
 		       ctrl_data->speed);
-	fprintf(stderr, "a52: offset reset\n");
+	//DNOTE("offset reset\n");
       }
     }
 
@@ -534,7 +617,7 @@ int get_q()
       if(TIME_S(time_offset) > 10) {
 	TIME_S(time_offset) = 0;
 	TIME_SS(time_offset) = 0;
-	//fprintf(stderr, "more than 10 secs in audio output buffer, somethings wrong?\n");
+	//DNOTE("more than 10 secs in audio output buffer, somethings wrong?\n");
       }
     }
 
@@ -584,15 +667,13 @@ int get_q()
     q_head->writer_requests_notification = 0;
     ev.type = MsgEventQNotify;
     if(MsgSendEvent(msgq, q_head->writer, &ev, 0) == -1) {
-      fprintf(stderr, "a52: couldn't send notification\n");
+      WARNING("couldn't send notification\n");
     }
   }
 
   return 0;
 }
 
-
-
 static clocktime_t a52_decode_data(uint8_t *start, uint8_t *end) {
   static a52_state_t state;
   
@@ -609,21 +690,20 @@ static clocktime_t a52_decode_data(uint8_t *start, uint8_t *end) {
   
   static int sample_rate;
   static int flags;
-  int bit_rate;
   int print_skip = 0, print_error = 0;
   
   while(start < end) {
     *bufptr++ = *start++;
     if(bufptr == bufpos) {
       if(bufpos == buf + 7) {
-	int length;
+	int coded_flags, bit_rate, length;
 	
 	if(print_error) {
-	  fprintf(stderr, "a52dec: error while decoding, restarting\n");
+	  DNOTE("A/52 error while decoding, restarting\n");
 	  print_error = 0;
 	}
 
-	length = a52_syncinfo(buf, &flags, &sample_rate, &bit_rate);
+	length = a52_syncinfo(buf, &coded_flags, &sample_rate, &bit_rate);
 	if(!length) {
 	  print_skip = 1;
 	  for(bufptr = buf; bufptr < buf + 6; bufptr++)
@@ -636,31 +716,55 @@ static clocktime_t a52_decode_data(uint8_t *start, uint8_t *end) {
 	int i;	
 	if(print_skip) {
 	  print_skip = 0;
-	  fprintf(stderr, "a52dec: Discarded data to find a valid frame\n");
+	  DNOTE("skipped data to find a valid frame\n");
 	}
+ 	
+	/* Verify or set the sample rate and retrive flags, level and bias */
 	if(ao_setup(output, sample_rate, &flags, &level, &bias)) {
-	  fprintf(stderr, "ao_setup() error\n");
-	  goto error;
+	  DNOTE("ao_setup() error\n");
+          open_output(get_speaker_flags());
+	  if(ao_setup(output, sample_rate, &flags, &level, &bias)) {
+	    goto error;
+	    //exit(1);
+	  }
+	  speaker_flags = flags;
+	  DNOTE("ao_setup() averted\n");
 	}
+	
+	if(speaker_flags == -1)
+	  speaker_flags = flags;
+	
+	flags = speaker_flags;
 	flags |= A52_ADJUST_LEVEL;
 	memset(&state, 0, sizeof(a52_state_t));
+	/* flags (speaker) [in/out] level [in/out] bias [in] */
 	if(a52_frame(&state, buf, &flags, &level, bias)) {
-	  fprintf(stderr, "a52_frame() error\n");
+	  DNOTE("a52_frame() error\n");
 	  goto error;
 	}
 
 	if(disable_dynrng)
 	  a52_dynrng(&state, NULL, NULL);
 	for(i = 0; i < 6; i++) {
-
 	  if(a52_block(&state, samples)) {
-	    fprintf(stderr, "a52_block() error\n");
+	    DNOTE("a52_block() error\n");
 	    goto error;
 	  }
-
+	  /* flags (output from decoder) verified by ao_play */
 	  if(ao_play(output, flags, samples)) {
-	    fprintf(stderr, "ao_play() error\n");
-	    goto error;
+	    int flags2;
+	    sample_t level2, bias2; // Dummy parameters
+	    DNOTE("ao_play() error\n");
+	    /* re-open... */
+	    open_output(get_speaker_flags());
+	    /* re-setup (set sample rate) */
+	    if(ao_setup(output, sample_rate, &flags2, &level2, &bias2))
+	      goto error;
+	    speaker_flags = flags;
+	    /* set the output mode to what ever flags is (hopefully) */
+	    if(ao_play(output, flags, samples))
+	      goto error;
+	    DNOTE("ao_play() averted\n");
 	  }
 	  blocks++;
 	}
