@@ -36,6 +36,7 @@
 #include "common.h"
 #include "queue.h"
 #include "timemath.h"
+#include "debug_print.h"
 
 #ifndef SHM_SHARE_MMU
 #define SHM_SHARE_MMU 0
@@ -46,12 +47,12 @@
 #define siginfo_t void
 #endif
 
-int create_msgq();
+int create_msgq(void);
 int init_decoder(char *msgqid_str, char *decoderstr);
 int get_buffer(int size, shm_bufinfo_t *bufinfo);
 int create_q(int nr_of_elems, int buf_shmid, 
 	     MsgEventClient_t writer, MsgEventClient_t reader);
-int create_ctrl_data();
+int create_ctrl_data(void);
 int register_stream(uint8_t stream_id, uint8_t subtype);
 
 static void handle_events(MsgEventQ_t *q, MsgEvent_t *ev);
@@ -59,9 +60,10 @@ static void handle_events(MsgEventQ_t *q, MsgEvent_t *ev);
 void int_handler(int sig);
 void sigchld_handler(int sig, siginfo_t *info, void* context);
 
-void remove_q_shm();
+void remove_q_shm(void);
 void add_q_shmid(int shmid);
-void destroy_msgq();
+void remove_q_shmid(int shmid);
+void destroy_msgq(void);
 
 void add_to_pidlist(pid_t pid);
 int remove_from_pidlist(pid_t pid);
@@ -101,7 +103,8 @@ static int child_killed = 0;
 
 void usage(void)
 {
-  fprintf(stderr, "Usage: %s [-h] [-a <ac3_stream#>] [-m <mpeg_audio_stream#>] [-p <pcm_audio_stream#>] [-v <mpeg_video_stream#>] [-s <subpicture_stream#>] [-n] [-f <fps>] [-r <#ouput_bufs>] [-o <file_offset>] [-d <videodebug_level>] [-D <demuxdebug_level>] <input file>\n", program_name);
+  fprintf(stderr, "Usage: %s [-h] [-u cli|gui] [<path>]", program_name);
+  //"[-a <ac3_stream#>] [-m <mpeg_audio_stream#>] [-p <pcm_audio_stream#>] [-v <mpeg_video_stream#>] [-s <subpicture_stream#>] [-n] [-f <fps>] [-r <#ouput_bufs>] [-o <file_offset>] [-d <videodebug_level>] [-D <demuxdebug_level>] <input file>\n", program_name);
 }
 
 
@@ -156,7 +159,7 @@ static int nr_caps = 0;
 int register_capabilities(MsgEventClient_t client, int caps, cap_state_t state)
 {
   if(nr_caps >= 20) {
-    fprintf(stderr, "ctrl: WARNING more than 20 capabilities registered\n");
+    WARNING("more than 20 capabilities registered\n");
   }
   nr_caps++;
   caps_array = realloc(caps_array, sizeof(caps_t)*nr_caps);
@@ -175,21 +178,29 @@ int search_capabilities(int caps, MsgEventClient_t *client, int *ret_caps,
   if(client != NULL) {
     *client = CLIENT_NONE;
   }
-  fprintf(stderr, "searching cap: %d\n", caps);
+#if DEBUG
+  DNOTE("searching cap: %d\n", caps);
+#endif
   
   for(n = 0; n < nr_caps; n++) {
     if((caps_array[n].caps & caps) == caps) {
       nr++;
       if(client != NULL) {
 	*client = caps_array[n].client;
-        fprintf(stderr, "found capclient: %ld\n", *client);
+#if DEBUG
+        DNOTE("found capclient: %ld\n", *client);
+#endif
       }
       if(ret_caps != NULL) {
 	*ret_caps = caps_array[n].caps;
-	fprintf(stderr, "found cap: %x\n", *ret_caps);
+#if DEBUG
+	DNOTE("found cap: %x\n", *ret_caps);
+#endif
       }
 
-      fprintf(stderr, "state cap: %d\n", caps_array[n].state);
+#if DEBUG
+      DNOTE("state cap: %d\n", caps_array[n].state);
+#endif
 
       if(ret_state != NULL) {
 	*ret_state = caps_array[n].state;
@@ -308,7 +319,7 @@ static char *capability_to_decoderstr(int capability, int *ret_capability)
 
 static void cleanup(void)
 {
-  fprintf(stderr, "ctrl: waiting for children to really die\n"); 
+  //DNOTE("waiting for children to really die\n"); 
   
   while(sleep(2)); // Continue sleeping if interupted 
 
@@ -324,8 +335,10 @@ int request_capability(MsgEventQ_t *q, int cap,
   MsgEvent_t r_ev;
   char *decodername;
   cap_state_t state = 0;
-  fprintf(stderr, "ctrl: _MsgEventQReqCapability\n");
 
+#if DEBUG
+  DNOTE("_MsgEventQReqCapability\n");
+#endif
   
   if(!search_capabilities(cap, capclient, retcaps, &state)) {
     int fullcap;
@@ -337,9 +350,7 @@ int request_capability(MsgEventQ_t *q, int cap,
 			    fullcap,
 			    CAP_started);
       
-      fprintf(stderr, "ctrl: starting decoder %d %s\n",
-	      fullcap,
-	      decodername);
+    //DNOTE("starting decoder %d %s\n", fullcap, decodername);
       init_decoder(msgqid_str, decodername);
     }
     
@@ -361,7 +372,9 @@ int request_capability(MsgEventQ_t *q, int cap,
   }
   
   if(state == CAP_running) {
-    fprintf(stderr, "ctrl: sending ctrldata\n");
+#if DEBUG
+    DNOTE("sending ctrldata\n");
+#endif
     r_ev.type = MsgEventQCtrlData;
     r_ev.ctrldata.shmid = ctrl_data_shmid;
     
@@ -369,7 +382,7 @@ int request_capability(MsgEventQ_t *q, int cap,
     
     return 1;
   } else {
-    fprintf(stderr, "ctrl: didn't find capability\n");
+    ERROR("didn't find capability\n");
     return 0;
   }
 }
@@ -385,13 +398,17 @@ static void handle_events(MsgEventQ_t *q, MsgEvent_t *ev)
   
   switch(ev->type) {
   case MsgEventQInitReq:
-    fprintf(stderr, "ctrl: _MsgEventQInitReq, new_id: %d\n", next_client_id);
+#if DEBUG
+    DNOTE("_MsgEventQInitReq, new_id: %d\n", next_client_id);
+#endif
     ev->type = MsgEventQInitGnt;
     ev->initgnt.newclientid = next_client_id++;
     MsgSendEvent(q, CLIENT_UNINITIALIZED, ev, 0);
     break;
   case MsgEventQRegister:
-    fprintf(stderr, "ctrl: _MsgEventQRegister\n");
+#if DEBUG
+    DNOTE("_MsgEventQRegister\n");
+#endif
     register_capabilities(ev->registercaps.client,
 			  ev->registercaps.capabilities,
 			  CAP_running);
@@ -416,8 +433,10 @@ static void handle_events(MsgEventQ_t *q, MsgEvent_t *ev)
     {
       shm_bufinfo_t bufinfo;
       
-      fprintf(stderr, "ctrl: _got request for buffer size %d\n",
+#if DEBUG
+      DNOTE("_got request for buffer size %d\n",
 	      ev->reqbuf.size);
+#endif
       if(get_buffer(ev->reqbuf.size, &bufinfo) == -1) {
 	bufinfo.shmid = -1;
       }
@@ -428,13 +447,26 @@ static void handle_events(MsgEventQ_t *q, MsgEvent_t *ev)
       MsgSendEvent(q, ev->reqbuf.client, &s_ev, 0);
     }
     break;
+  case MsgEventQDestroyBuf:
+    {
+      //DNOTE("_got destroy buffer shmid %d\n", ev->destroybuf.shmid);
+      remove_q_shmid(ev->destroybuf.shmid);
+    }
+    break;
+  case MsgEventQDestroyQ:
+    {
+      //DNOTE("_got destroy Q shmid %d\n", ev->detachq.q_shmid);
+      remove_q_shmid(ev->detachq.q_shmid);
+    }
+    break;
   case MsgEventQReqStreamBuf:
     {
       int shmid;
       cap_state_t state = 0;
-      fprintf(stderr, "ctrl: _new stream %x, %x\n",
-	      ev->reqstreambuf.stream_id,
-	      ev->reqstreambuf.subtype);
+      
+      DNOTE("_new stream %x, %x\n",
+	    ev->reqstreambuf.stream_id,
+	    ev->reqstreambuf.subtype);
       
       if(register_stream(ev->reqstreambuf.stream_id,
 			 ev->reqstreambuf.subtype)) {
@@ -457,7 +489,7 @@ static void handle_events(MsgEventQ_t *q, MsgEvent_t *ev)
 					       ev->reqstreambuf.subtype);
 	  
 	  if((capability & VIDEO_OUTPUT) || (capability & DECODE_DVD_SPU)) {
-	    fprintf(stderr, "****ctrl: registered VO or SPU started\n");
+	    DNOTE("registered VO or SPU started\n");
 	  }
 	  if(capability == DECODE_DVD_SPU) {
 	    register_capabilities(0,
@@ -468,11 +500,9 @@ static void handle_events(MsgEventQ_t *q, MsgEvent_t *ev)
 				  capability,
 				  CAP_started);
 	  }
-	  fprintf(stderr, "ctrl: starting decoder %d %s\n", capability,
-		  decodername);
+ // DNOTE("starting decoder %d %s\n", capability, decodername);
 	  init_decoder(msgqid_str, decodername);
-	  fprintf(stderr, "ctrl: started decoder %d\n", capability);
-	 
+ // DNOTE("started decoder %d\n", capability);
 	}
 	
 	while(!search_capabilities(capability, &rcpt, NULL, &state) ||
@@ -494,7 +524,9 @@ static void handle_events(MsgEventQ_t *q, MsgEvent_t *ev)
 	
 	// send ctrl_data shm: let client know where the timebase
 	// data is
-	fprintf(stderr, "ctrl: sending ctrldata\n");
+#if DEBUG
+	DNOTE("sending ctrldata\n");
+#endif
 	s_ev.type = MsgEventQCtrlData;
 	s_ev.ctrldata.shmid = ctrl_data_shmid;
 	
@@ -551,8 +583,10 @@ static void handle_events(MsgEventQ_t *q, MsgEvent_t *ev)
     {
       int shmid;
       cap_state_t state;
-      fprintf(stderr, "ctrl: _new pic q\n");
-      
+
+#if DEBUG
+      DNOTE("_new pic q\n");
+#endif
       // check if we have a decoder
       
       //TODO check which decoder handles the stream and start it
@@ -563,13 +597,13 @@ static void handle_events(MsgEventQ_t *q, MsgEvent_t *ev)
       //
       if(!search_capabilities(VIDEO_OUTPUT, &rcpt, NULL, NULL)) {
 	
-	fprintf(stderr, "****ctrl: registered VO|SPU started\n");
+	DNOTE("registered VO|SPU started\n");
 	register_capabilities(0,
 			      VIDEO_OUTPUT | DECODE_DVD_SPU,
 			      CAP_started);
 	
 	init_decoder(msgqid_str, getenv("DVDP_VIDEO_OUT"));
-	fprintf(stderr, "ctrl: started video_out\n");
+	//DNOTE("started video_out\n");
       }
       while(!search_capabilities(VIDEO_OUTPUT, &rcpt, NULL, &state) ||
 	    (state != CAP_running)) {
@@ -585,13 +619,15 @@ static void handle_events(MsgEventQ_t *q, MsgEvent_t *ev)
 	}
 	handle_events(q, &r_ev);
       }
-      fprintf(stderr, "ctrl: got capability video_out\n");
+      DNOTE("got capability video_out\n");
       
       // we now have a decoder running ready to decode the stream
       
       // send ctrl_data shm: let client know where the timebase
       // data is
-      fprintf(stderr, "ctrl: sending ctrldata\n");
+#if DEBUG
+      DNOTE("sending ctrldata\n");
+#endif
       s_ev.type = MsgEventQCtrlData;
       s_ev.ctrldata.shmid = ctrl_data_shmid;
 	
@@ -612,9 +648,10 @@ static void handle_events(MsgEventQ_t *q, MsgEvent_t *ev)
       s_ev.type = MsgEventQGntPicBuf;
       s_ev.gntpicbuf.q_shmid = shmid;
       
-      fprintf(stderr, "ctrl: create_q, q_shmid: %d picture_buf_shmid: %d\n",
-	      shmid,  ev->reqpicbuf.data_buf_shmid);
-      
+#if DEBUG
+      DNOTE("create_q, q_shmid: %d picture_buf_shmid: %d\n",
+	    shmid,  ev->reqpicbuf.data_buf_shmid);
+#endif      
       MsgSendEvent(q, ev->reqpicbuf.client, &s_ev, 0);
       
       // connect the picbuf  to the reader
@@ -631,19 +668,16 @@ static void handle_events(MsgEventQ_t *q, MsgEvent_t *ev)
       clocktime_t rt;
       
       clocktime_get(&rt);
-      fprintf(stderr, "ctrl: _MsgEventQSpeed\n");
-      fprintf(stderr, "ctrl: speed: %.2f\n", ev->speed.speed);
-
+#if DEBUG
+      DNOTE("_MsgEventQSpeed\n");
+      DNOTE("speed: %.2f\n", ev->speed.speed);
+#endif
       ctrl_data->speed = ev->speed.speed;
       
       
 
       // send speed event to syncmasters
-      
       {
-	fprintf(stderr, "ctrl: new speed\n");
-	
-	
 	// TODO get decoders that do sync...
 	//
 	if(search_capabilities(DECODE_AC3_AUDIO, &rcpt, NULL, NULL)) {
@@ -661,7 +695,7 @@ static void handle_events(MsgEventQ_t *q, MsgEvent_t *ev)
     }
     break;
   default:
-    fprintf(stderr, "ctrl: handle_events: msgtype %d not handled\n",
+    WARNING("handle_events: notice, msgtype %d not handled\n",
 	    ev->type);
     break;
   }
@@ -673,12 +707,20 @@ int main(int argc, char *argv[])
   int c;
   MsgEventQ_t q;
   MsgEvent_t ev;
+  
+  for(c = strlen(argv[0])-1; c > 0; c--) {
+    if(argv[0][c] == '/') {
+      c++;
+      break;
+    }
+  }
+  program_name = &argv[0][c];
 
   memset(&sig, 0, sizeof(struct sigaction));
   sig.sa_handler = int_handler;
   sig.sa_flags = 0;
   if(sigaction(SIGINT, &sig, NULL) == -1) {
-    perror("sigaction1");
+    perror("ctrl: failed to set sigaction SIGINT handler");
   }
 
 #if defined(SA_SIGINFO)
@@ -688,14 +730,14 @@ int main(int argc, char *argv[])
   sig.sa_handler = (sig_t)sigchld_handler;
 #endif
   if(sigaction(SIGCHLD, &sig, NULL) == -1) {
-    perror("sigaction2");
+    perror("ctrl: failed to set sigaction SIGCHL handler");
   }
   
   
-  program_name = argv[0];
-  
-  while((c = getopt(argc, argv, "na:v:s:m:f:r:o:p:d:u:t:h?")) != EOF) {
+  //"na:v:s:m:f:r:o:p:d:u:t:h?"
+  while((c = getopt(argc, argv,"u:h?" )) != EOF) {
     switch(c) {
+#if 0
     case 'a':
       ac3_audio_stream = atoi(optarg);
       break;
@@ -717,9 +759,6 @@ int main(int argc, char *argv[])
     case 'n':
       nav_stream = 1;
       break;
-    case 'u':
-      ui = optarg;
-      break;
     case 'f':
       framerate = optarg;
       break;
@@ -735,6 +774,10 @@ int main(int argc, char *argv[])
     case 'D':
       demux_debug = optarg;
       break;      
+#endif
+    case 'u':
+      ui = optarg;
+      break;
     case 'h':
     case '?':
       usage();
@@ -754,29 +797,25 @@ int main(int argc, char *argv[])
     input_file = NULL;
   }
   
-  
-  fprintf(stderr, "%s %s\n", PACKAGE, VERSION);
-  
+  /* Print the version info sot that we get it with bug reports. */
+  NOTE("%s %s\n", PACKAGE, VERSION);
   
   ctrl_data_shmid = create_ctrl_data();
   
-  /* create msgq */
-  
+  /* create msgq */  
   create_msgq();
-  
-
   sprintf(msgqid_str, "%d", msgqid);
   
-  fprintf(stderr, "msgid: %d\n", msgqid);
-  
+#if DEBUG
+  DNOTE("msgid: %d\n", msgqid);  
   {
     struct msqid_ds msgqinfo;
     
     msgctl(msgqid, IPC_STAT, &msgqinfo);
     
-    fprintf(stderr, "max_bytes: %ld\n", (long)msgqinfo.msg_qbytes);
-
+    DNOTE("max_bytes: %ld\n", (long)msgqinfo.msg_qbytes);
   }
+#endif
 
   q.msqid = msgqid;
   q.mtype = CLIENT_RESOURCE_MANAGER;
@@ -788,7 +827,7 @@ int main(int argc, char *argv[])
     } else if(!strcmp("gui", ui)) {
       request_capability(&q, UI_DVD_GUI, &ui_client, NULL);
     } else {
-      fprintf(stderr, "**ctrl: no ui specified\n");
+      FATAL("no ui specified\n");
       cleanup_and_exit();
     }
   }
@@ -833,19 +872,19 @@ int init_decoder(char *msgqid_str, char *decoderstr)
   int n;
   
   if(decode_path == NULL) {
-    fprintf(stderr, "*ctrl: init_decoder(): decoder not set\n");
-    return(-1);
+    ERROR("init_decoder(): decoder not set\n");
+    return -1;
   }
   
   if((decode_name = strrchr(decode_path, '/')+1) == NULL) {
     decode_name = decode_path;
   }
   if(decode_name > &decode_path[strlen(decode_path)]) {
-    fprintf(stderr, "*ctrl: init_decoder(): illegal file name?\n");
+    ERROR("init_decoder(): illegal file name?\n");
     return -1;
   }
   
-  fprintf(stderr, "ctrl: init_decoder(): %s\n", decode_name);
+  //DNOTE("init_decoder(): %s\n", decode_name);
 
   //starting_decoder = 1;
 
@@ -882,8 +921,8 @@ int init_decoder(char *msgqid_str, char *decoderstr)
     eargv[n++] = NULL;
     
     if(execv(decode_path, eargv) == -1) {
-      perror("ctrl: init_decoder(): execv()");
-      fprintf(stderr, "ctrl: init_decoder(): path: %s\n", decode_path);
+      FATAL("init_decoder(): path: %s\n", decode_path);
+      perror("execv");
     }
     exit(1);
     break;
@@ -900,10 +939,9 @@ int init_decoder(char *msgqid_str, char *decoderstr)
   }
   
 
-  fprintf(stderr, "Started %s with pid %ld\n", decoderstr, (long)pid);
+  DNOTE("Started %s with pid %ld\n", decoderstr, (long)pid);
   //  starting_decoder = 0;
   return pid;
-
 }
 
 
@@ -1073,25 +1111,23 @@ int register_stream(uint8_t stream_id, uint8_t subtype)
 }
 
 
-int create_msgq()
+int create_msgq(void)
 {
-  
   if((msgqid = msgget(IPC_PRIVATE, IPC_CREAT | 0600)) == -1) {
-    perror("msgget msgqid");
-    exit(-1);
+    perror("*ctrl: msgget ipc_creat failed");
+    exit(1);
   }
 
   return 0;
 }  
 
 
-void destroy_msgq()
+void destroy_msgq(void)
 {
-  if(msgctl(msgqid, IPC_RMID, NULL) == -1) {
-    perror("msgctl ipc_rmid");
+  if(msgctl(msgqid, IPC_RMID, NULL) != 0) {
+    perror("*ctrl: msgctl ipc_rmid failed");
   }
   msgqid = -1;
-
 }
 
 
@@ -1099,11 +1135,10 @@ int get_buffer(int size, shm_bufinfo_t *bufinfo)
 {
   int shmid;
   
-  fprintf(stderr, "get_buffer\n");
   if((shmid = shmget(IPC_PRIVATE,
 		     size,
 		     IPC_CREAT | 0600)) == -1) {
-    perror("get_buffer(), shmget()");
+    perror("*ctrl: get_buffer, shmget failed");
     return -1;
   }
 
@@ -1126,22 +1161,19 @@ int create_q(int nr_of_elems, int buf_shmid,
   q_elem_t *q_elems;
   int n;
 
-  fprintf(stderr, "create_q\n");
-  fprintf(stderr, "shmget\n");
   if((shmid = shmget(IPC_PRIVATE,
 		     sizeof(q_head_t) + nr_of_elems*sizeof(q_elem_t),
 		     IPC_CREAT | 0600)) == -1) {
-    perror("create_q(), shmget()");
+    perror("*ctrl: create_q, shmget failed");
     return -1;
   }
   
   
-  fprintf(stderr, "shmat\n");
   if((shmaddr = shmat(shmid, NULL, SHM_SHARE_MMU)) == (void *)-1) {
-    perror("create_q(), shmat()");
+    perror("*ctrl: create_q, shmat failed");
     
-    if(shmctl(shmid, IPC_RMID, NULL) == -1) {
-      perror("ipc_rmid");
+    if(shmctl(shmid, IPC_RMID, NULL) != 0) {
+      perror("*ctrl: create_q, shmctl ipc_rmid faild");
     }
     return -1;
   }
@@ -1166,8 +1198,8 @@ int create_q(int nr_of_elems, int buf_shmid,
   for(n = 0; n < nr_of_elems; n++) {
     q_elems[n].in_use = 0;
   }
-  if(shmdt(shmaddr) == -1) {
-    perror("create_q(), shmdt()");
+  if(shmdt(shmaddr) != 0) {
+    perror("*ctrl: create_q, shmdt failed");
   }
   return shmid;
 }
@@ -1175,7 +1207,7 @@ int create_q(int nr_of_elems, int buf_shmid,
 
 
 
-int create_ctrl_data()
+int create_ctrl_data(void)
 {
   int shmid;
   char *shmaddr;
@@ -1187,15 +1219,15 @@ int create_ctrl_data()
 		     sizeof(ctrl_data_t)+
 		     nr_of_offsets*sizeof(ctrl_time_t),
 		     IPC_CREAT | 0600)) == -1) {
-    perror("create_ctrl_data(), shmget()");
+    perror("*ctrl: create_ctrl_data, shmget failed");
     return -1;
   }
   
   if((shmaddr = shmat(shmid, NULL, SHM_SHARE_MMU)) == (void *)-1) {
-    perror("create_ctrl_data(), shmat()");
+    perror("*ctrl: create_ctrl_data, shmat failed");
     
     if(shmctl(shmid, IPC_RMID, NULL) == -1) {
-      perror("ipc_rmid");
+      perror("*ctrl: create_ctrl_data, shmctl ipc_rmid");
     }
     
     return -1;
@@ -1214,7 +1246,6 @@ int create_ctrl_data()
   }
   
   return shmid;
-  
 }
 
 
@@ -1226,7 +1257,7 @@ void add_q_shmid(int shmid)
   nr_shmids++;
   
   if((shm_ids = (int *)realloc(shm_ids, sizeof(int)*nr_shmids)) == NULL) {
-    perror("realloc");
+    perror("*ctrl: add_q_shmid, realloc failed");
     nr_shmids--;
     return;
   }
@@ -1236,14 +1267,32 @@ void add_q_shmid(int shmid)
 }
 
 
-void remove_q_shm()
+void remove_q_shmid(int shmid)
+{
+  int n;
+  
+  for(n = 0; n < nr_shmids; n++) {
+    if(shm_ids[n] == shmid) {
+      DNOTE("removing shmid: %d\n", shm_ids[n]);
+      if(shmctl(shm_ids[n], IPC_RMID, NULL) == -1) {
+	perror("ipc_rmid");
+      }
+      shm_ids[n] = -1;
+    }
+  }
+}
+
+
+void remove_q_shm(void)
 {
   int n;
 
   for(n = 0; n < nr_shmids; n++) {
-    fprintf(stderr, "removing shmid: %d\n", shm_ids[n]);
-    if(shmctl(shm_ids[n], IPC_RMID, NULL) == -1) {
-      perror("ipc_rmid");
+    if(shm_ids[n] != -1) {
+      DNOTE("removing shmid: %d\n", shm_ids[n]);
+      if(shmctl(shm_ids[n], IPC_RMID, NULL) == -1) {
+	perror("ctrl: ipc_rmid");
+      }
     }
   }
   nr_shmids = 0;
@@ -1254,10 +1303,9 @@ void remove_q_shm()
 
 void cleanup_and_exit(void)
 {
-  fprintf(stderr, "ctrl: cleaning up\n");
   remove_q_shm();
   destroy_msgq();
-  fprintf(stderr, "ctrl: exiting\n");
+  NOTE("exiting\n");
   exit(0);
 }
 
@@ -1275,16 +1323,16 @@ void int_handler(int sig)
   int n;
   pid_t pid;
   
-  fprintf(stderr, "Caught signal %d, cleaning up\n", sig);
+  DNOTE("Caught signal %d, cleaning up\n", sig);
 
   for(n = 0; n < num_pids; n++) {
     if((pid = pidlist[n]) != -1) {
       child_killed = 1;
-      fprintf(stderr, "killing child: %ld\n", (long)pid);
+      DNOTE("killing child: %ld\n", (long)pid);
       kill(pid, SIGINT);
     }
     if(!child_killed) {
-      fprintf(stderr, "ctrl: All children dead\n");
+      DNOTE("All children dead\n");
       child_killed = 1;
     }
   }
@@ -1318,46 +1366,49 @@ void sigchld_handler(int sig, siginfo_t *info, void* context)
   pid_t wpid, pid;
   
 #if defined(SA_SIGINFO)
-  fprintf(stderr, "ctrl: sigchldhandler got signal %d\n", info->si_signo);
-  if(info->si_errno) {
-    fprintf(stderr, "ctrl: error: %s\n", strerror(info->si_errno));
+  if(info->si_signo != SIGCHLD) {
+    ERROR("sigchldhandler got signal %d\n", info->si_signo);
+    return;
   }
-  fprintf(stderr, "si_code: %d\n", info->si_code);
+  if(info->si_errno) {
+    DNOTE("error: %s\n", strerror(info->si_errno));
+  }
+  //DNOTE("si_code: %d\n", info->si_code);
 
-  fprintf(stderr, "si_pid: %ld\n", (long)info->si_pid);
+  //DNOTE("si_pid: %ld\n", (long)info->si_pid);
   
   switch(info->si_code) {
   case CLD_STOPPED:
-    fprintf(stderr, "ctrl: child: %ld stopped\n", (long)info->si_pid);
+    DNOTE("child: %ld stopped\n", (long)info->si_pid);
     break;
   case CLD_CONTINUED:
-    fprintf(stderr, "ctrl: child: %ld continued\n", (long)info->si_pid);
+    DNOTE("child: %ld continued\n", (long)info->si_pid);
     break;
   case CLD_KILLED:
-    fprintf(stderr, "ctrl: child: %ld killed\n", (long)info->si_pid);
+    DNOTE("child: %ld killed\n", (long)info->si_pid);
     died = 1;
     break;
   case CLD_DUMPED:
-    fprintf(stderr, "ctrl: child: %ld dumped\n", (long)info->si_pid);
+    DNOTE("child: %ld dumped\n", (long)info->si_pid);
     died = 1;
     break;
   case CLD_TRAPPED:
-    fprintf(stderr, "ctrl: child: %ld trapped\n", (long)info->si_pid);
+    DNOTE("child: %ld trapped\n", (long)info->si_pid);
     died = 1;
     break;
   case CLD_EXITED:
-    fprintf(stderr, "ctrl: child: %ld exited with %d\n",
+    DNOTE("child: %ld exited with %d\n",
 	    (long)info->si_pid, info->si_status);
     died = 1;
     break;
 #if defined(SI_NOINFO)  // Solaris only
-   case SI_NOINFO:
-    fprintf(stderr, "ctrl: sigchld with no info\n");
+  case SI_NOINFO:
+    DNOTE("pid %ld, sigchld: no info\n", (long)info->si_pid);
     break;
 #endif
   default:
-    fprintf(stderr, "**ctrl: unknown cause of sigchld, si_code: %d\n",
-	    info->si_code);
+    DNOTE("pid %ld, unknown sigchld si_code: %d\n",
+	    (long)info->si_pid, info->si_code);
     break;
   }
   wpid = info->si_pid;
@@ -1367,7 +1418,7 @@ void sigchld_handler(int sig, siginfo_t *info, void* context)
   
   while(1) {
     if((pid = waitpid(wpid, &stat_loc, WCONTINUED | WUNTRACED)) == -1) {
-      perror("ctrl: waitpid");
+      perror("ctrl: waitpid failed");
       switch(errno) {
       case EINTR:
 	continue;
@@ -1380,38 +1431,37 @@ void sigchld_handler(int sig, siginfo_t *info, void* context)
   
   if(WIFEXITED(stat_loc)) {
     died = 1;
-    fprintf(stderr, "pid: %ld exited with status: %d\n",
+    DNOTE("pid: %ld exited with status: %d\n",
 	    (long)pid, WEXITSTATUS(stat_loc));
   } else if(WIFSIGNALED(stat_loc)) {
     died = 1;
-    fprintf(stderr, "pid: %ld terminated on signal: %d\n",
+    DNOTE("pid: %ld terminated on signal: %d\n",
 	    (long)pid, WTERMSIG(stat_loc));
   } else if(WIFSTOPPED(stat_loc)) {
-    fprintf(stderr, "pid: %ld stopped on signal: %d\n",
+    DNOTE("pid: %ld stopped on signal: %d\n",
 	    (long)pid, WSTOPSIG(stat_loc));
   } else if(WIFCONTINUED(stat_loc)) {
-    fprintf(stderr, "pid: %ld continued\n", (long)pid);
+    DNOTE("pid: %ld continued\n", (long)pid);
   } else {
-    fprintf(stderr, "pid: %ld\n", (long)pid);
+    DNOTE("pid: %ld\n", (long)pid);
   }
   if(died) {
     int n;
     if(!remove_from_pidlist(pid)) {
-      fprintf(stderr, "pid died before registering\n");
+      DNOTE("pid died before registering\n");
     }
     for(n = 0; n < num_pids; n++) {
       if((pid = pidlist[n]) != -1) {
 	child_killed = 1;
-	fprintf(stderr, "killing child: %ld\n", (long)pid);
+	//DNOTE("killing child: %ld\n", (long)pid);
 	kill(pid, SIGINT);
       }
     }
     if(!child_killed) {
-      fprintf(stderr, "ctrl: all children dead\n");
+      DNOTE("all children dead\n");
       child_killed = 2;
     }
   }
-
 }
 
 
@@ -1419,12 +1469,16 @@ void slay_children(void)
 {
   int n;
   pid_t pid;
+  
   for(n = 0; n < num_pids; n++) {
     if((pid = pidlist[n]) != -1) {
       child_killed = 2;
-      fprintf(stderr, "slaying child: %ld\n", (long)pid);
+      DNOTE("slaying child: %ld\n", (long)pid);
       kill(pid, SIGKILL);
     }
   }
-  child_killed = 2;
+  if(!child_killed) {
+    DNOTE("No children left in pidlist\n");
+    child_killed = 2;
+  }
 }
