@@ -67,7 +67,7 @@ extern void exit_program(int exitcode) ATTRIBUTE_NORETURN;
 #if PRAGMA_NORETURN
 #pragma does_not_return (exit_program) 
 #endif
-extern void motion_comp();
+extern void motion_comp(void);
 extern void motion_comp_add_coeff(unsigned int i);
 extern int get_vlc(const vlc_table_t *table, char *func);
 
@@ -135,7 +135,7 @@ void reset_dc_dct_pred(void)
 }
 
 static
-void reset_PMV()
+void reset_PMV(void)
 {
   DPRINTFI(3, "Resetting PMV\n");
 
@@ -150,7 +150,7 @@ void reset_PMV()
 }
 
 static
-void reset_vectors()
+void reset_vectors(void)
 {
   DPRINTFI(3, "Resetting motion vectors\n");
   
@@ -303,7 +303,6 @@ void block_intra(unsigned int i)
     
     // mb.QFS[i] = sgn ? -f : f;
     mb.QFS[i] = (f ^ -sgn) + sgn;
-    
   }
   
   //cpc_count_usr_events(0);
@@ -618,7 +617,6 @@ void block_intra(unsigned int i)
       }
     }
     word = (cur_word << (64-left)) >> 40;
-    
   }
 
   //cpc_count_usr_events(0);
@@ -922,9 +920,7 @@ void motion_vectors(unsigned int s)
       /* Table 6-17 Meaning of frame_motion_type */
       switch(mb.modes.frame_motion_type) {
       case 0x0:
-	fprintf(stderr, "*** reserved prediction type\n");
-	exit_program(1);
-	break;
+	// Should never happen, value is checked when read in.
       case 0x1:
 	mb.prediction_type = PRED_TYPE_FIELD_BASED;
 	mb.motion_vector_count = 2;
@@ -955,9 +951,7 @@ void motion_vectors(unsigned int s)
     /* Table 6-18 Meaning of field_motion_type */
     switch(mb.modes.field_motion_type) {
     case 0x0:
-      fprintf(stderr, "*** reserved field prediction type\n");
-      exit_program(1);
-      break;
+      // Should never happen, value is checked when read in.
     case 0x1:
       mb.prediction_type = PRED_TYPE_FIELD_BASED;
       mb.motion_vector_count = 1;
@@ -1021,8 +1015,10 @@ int macroblock_modes(void)
     exit_program(1);
     // should not be tested / handled here
   }
-  
-  if(mb.modes.macroblock_type == VLC_FAIL) return -1;
+  if(mb.modes.macroblock_type == VLC_FAIL) {
+    fprintf(stderr, "*** invalid macroblock type\n");
+    return -1;
+  }
   
   DPRINTF(5, "spatial_temporal_weight_code_flag: %01x\n", 
 	  !!(mb.modes.macroblock_type & SPATIAL_TEMPORAL_WEIGHT_CODE_FLAG));
@@ -1041,9 +1037,17 @@ int macroblock_modes(void)
 	mb.modes.frame_motion_type = 0x2;
       } else {
 	mb.modes.frame_motion_type = GETBITS(2, "frame_motion_type");
+	if(mb.modes.frame_motion_type == 0x0) {
+	  fprintf(stderr, "*** invalid frame motion type\n");
+	  return -1;
+	}
       }
     } else {
       mb.modes.field_motion_type = GETBITS(2, "field_motion_type");
+      if(mb.modes.field_motion_type == 0x0) {
+	fprintf(stderr, "*** invalid field motion type\n");
+	return -1;
+      }
     }
   }
 
