@@ -47,12 +47,15 @@
 #include <sys/shm.h>
 
 #include "common.h"
+#include "debug_print.h"
 #include "video_types.h"
 #include "yuv2rgb.h"
 #include "screenshot.h"
 #include "wm_state.h"
-
+#include "display.h"
 #include "ffb_asm.h"
+#include "video_output.h"
+#include "video_output_parse_config.h"
 
 #define SPU
 #ifdef SPU
@@ -75,11 +78,15 @@ static XvImageFormatValues *xv_formats;
 static int xv_id;
 #endif /* HAVE_XV */
 
+
+extern char *program_name;
 //ugly hack
+extern data_q_t *cur_data_q;
+/*
 #ifdef HAVE_XV
 extern yuv_image_t *reserv_image;
 #endif
-
+*/
 /* needed for sun ffb2 */
 #include <sys/mman.h>
 
@@ -122,16 +129,10 @@ typedef struct {
 
 static window_info window;
 
-struct {
-  int width;
-  int height;
-  int horizontal_pixels;
-  int vertical_pixels;
-} dpy_size;
-
 static XVisualInfo vinfo;
 static XShmSegmentInfo shm_info;
 static Display *mydisplay = NULL;
+static int screen_nr;
 static GC mygc;
 static char title[100];
 static int color_depth, pixel_stride, mode;
@@ -141,8 +142,7 @@ static int screenshot = 0;
 static int screenshot_spu = 0;
 
 
-static int dpy_sar_frac_n;  /* Display (i.e. monitor) aspect. */
-static int dpy_sar_frac_d;  /*(set in display_init) */
+
 
 static struct {
   int zoom_n;              /* Zoom factor. */
@@ -177,9 +177,9 @@ extern MsgEventClient_t input_client;
 extern InputMask_t input_mask;
 
 /* All display modules shall define these functions */
-void display_init(yuv_image_t *picture_data,
-		  data_buf_head_t *picture_data_head,
-		  char *picture_buf_base);
+void display_init(int padded_width, int padded_height,
+		  int picture_buffer_shmid,
+		  char *picture_buffer_addr);
 void display(yuv_image_t *current_image);
 void display_poll(yuv_image_t *current_image);
 void display_exit(void);
@@ -261,19 +261,19 @@ static int xshm_errorhandler(Display *dpy, XErrorEvent *ev)
      * we assume that xshm doesn't work,
      * eg we are using a remote display
      */
-    fprintf(stderr, "req_code: %d\n", ev->request_code);
-    fprintf(stderr, "minor_code: %d\n", ev->minor_code);
-    fprintf(stderr, "error_code: %d\n", ev->error_code);
+    WARNING("req_code: %d\n", ev->request_code);
+    WARNING("minor_code: %d\n", ev->minor_code);
+    WARNING("error_code: %d\n", ev->error_code);
     
     use_xshm = 0;
-    fprintf(stderr, "*vo: Disabling Xshm\n");
+    WARNING("Disabling Xshm\n");
     return 0;
   } else {
     /* if we get another error we should handle it, 
      * so we give it to the previous errorhandler
      */
-    fprintf(stderr, "*vo: unexpected error serial: %lu, waited for: %lu\n",
-	    ev->serial, req_serial);
+    ERROR("unexpected error serial: %lu, waited for: %lu\n",
+	  ev->serial, req_serial);
     return prev_xerrhandler(dpy, ev);
   }
 }
@@ -284,9 +284,10 @@ static int xshm_errorhandler(Display *dpy, XErrorEvent *ev)
  * xv image and returns.
  *
  * The variable use_xv tells if Xv is used */
-static void display_init_xv(yuv_image_t *picture_data,
-			    data_buf_head_t *picture_data_head,
-			    char *picture_buf_base) 
+static void display_init_xv(int picture_buffer_shmid,
+			    char *picture_buffer_addr,
+			    int padded_width,
+			    int padded_height)
 {
 #ifdef HAVE_XV
   int i, j;
@@ -299,17 +300,20 @@ static void display_init_xv(yuv_image_t *picture_data,
 			    &xv_request_base, &xv_event_base, 
 			    &xv_error_base);
   if(result != Success) {
-    fprintf(stderr, "Xv not found\n");
+    WARNING("Xvideo extension not found\n");
     return;
   }
   
-  fprintf(stderr, "Found Xv extension, checking for suitable adaptors\n");
+  NOTE("Found Xv extension %d.%d, checking for suitable adaptors\n",
+       xv_version, xv_release);
+  
   /* Check for available adaptors */
   result = XvQueryAdaptors(mydisplay, DefaultRootWindow (mydisplay), 
 			   &xv_num_adaptors, &xv_adaptor_info);
-  if(result != Success)
+  if(result != Success) {
+    WARNING("No Xv adaptors found\n");
     return;
-  
+  }
       
   /* Check adaptors */
   for(i = 0; i < xv_num_adaptors; i++) {
@@ -320,16 +324,12 @@ static void display_init_xv(yuv_image_t *picture_data,
       continue;
     
     xv_port = xv_adaptor_info[i].base_id;
-    fprintf(stderr, "Found adaptor \"%s\" checking for suitable formats\n",
-	    xv_adaptor_info[i].name);
       
     /* Check image formats of adaptor */
     xv_formats = XvListImageFormats(mydisplay, xv_port, &xv_num_formats);
     for(j = 0; j < xv_num_formats; j++) {
       if(xv_formats[j].id == 0x32315659) { /* YV12 */
 	//if(xv_formats[j].id == 0x30323449) { /* I420 */
-	fprintf(stderr, "Found image format \"%s\", using it\n", 
-		xv_formats[j].guid);
 	xv_id = xv_formats[j].id;
 	break;
       } 
@@ -338,12 +338,13 @@ static void display_init_xv(yuv_image_t *picture_data,
     if(j == xv_num_formats)
       continue;
       
-    fprintf(stderr, "Using Xvideo port %li for hw scaling\n", xv_port);
+    NOTE("Xv adaptor \"%s\" port %li image format %i\n",
+	 xv_adaptor_info[i].name, xv_port, xv_id);
       
     /* Allocate XvImages */
     xv_image = XvShmCreateImage(mydisplay, xv_port, xv_id, NULL,
-				picture_data->info->picture.padded_width,
-				picture_data->info->picture.padded_height, 
+				padded_width,
+				padded_height, 
 				&shm_info);
     
     /* Got an Image? */
@@ -351,18 +352,18 @@ static void display_init_xv(yuv_image_t *picture_data,
       continue;
     
     /* Test and see if we really got padded_width x padded_height */
-    if(xv_image->width != picture_data->info->picture.padded_width ||
-       xv_image->height != picture_data->info->picture.padded_height) {
-      fprintf(stderr, "vo: XvShmCreateImage got size: %d x %d\n",
-	      xv_image->width, xv_image->height);
+    if(xv_image->width != padded_width ||
+       xv_image->height != padded_height) {
+      FATAL("XvShmCreateImage got size: %d x %d\n",
+	    xv_image->width, xv_image->height);
       exit(1);
     }
     
-    shm_info.shmid = picture_data_head->shmid;
-    shm_info.shmaddr = picture_buf_base;
+    shm_info.shmid = picture_buffer_shmid;
+    shm_info.shmaddr = picture_buffer_addr;
     
     /* Set the data pointer to the decoders picture segment. */  
-    xv_image->data = picture_data->y;
+    //    xv_image->data = picture_data->y;
     shm_info.readOnly = True;
     
     /* make sure we don't have any unhandled errors */
@@ -414,18 +415,19 @@ static void display_init_xshm()
 				  ZPixmap, NULL, &shm_info,
 				  scale.image_width,
 				  scale.image_height);
-  
+
   /* Got an Image? */
   if(window.ximage == NULL) {
-    fprintf(stderr, "vo: XShmCreateImage failed\n");
+    // TODO: should revert to normal X in this case
+    FATAL("XShmCreateImage failed\n");
     exit(1);
   }
   
   /* Test and see if we really got padded_width x padded_height */
   if(window.ximage->width != scale.image_width ||
      window.ximage->height != scale.image_height) {
-    fprintf(stderr, "vo: XShmCreateImage got size: %d x %d\n",
-	    window.ximage->width, window.ximage->height);
+    FATAL("XShmCreateImage got size: %d x %d\n",
+	  window.ximage->width, window.ximage->height);
     exit(1);
   }
   
@@ -435,8 +437,9 @@ static void display_init_xshm()
 			  window.ximage->height, 
 			  IPC_CREAT | 0777);
   
-  if(shm_info.shmid < 0) {
-    fprintf(stderr, "vo: shmget failed\n");
+  if(shm_info.shmid == -1) {
+    FATAL("display_init_xshm\n");
+    perror("shmget");
     exit(1);
   }
   
@@ -444,14 +447,14 @@ static void display_init_xshm()
   shm_info.shmaddr = (char *) shmat(shm_info.shmid, 0, 0);
   
   if(shm_info.shmaddr == ((char *) -1)) {
-    fprintf(stderr, "vo: shmat failed\n");
+    FATAL("display_init_xshm\n");
+    perror("shmat");
     exit(1);
   }
   
   /* Set the data pointer to the allocated segment. */  
   window.ximage->data = shm_info.shmaddr;
   shm_info.readOnly = False;
-  
 
   /* make sure we don't have any unhandled errors */
   XSync(mydisplay, False);
@@ -478,10 +481,6 @@ static void display_init_xshm()
   
   pixel_stride = window.ximage->bits_per_pixel;
 
-  
-  fprintf(stderr, "vo: pixel_stride: %d\n", pixel_stride);
-  
-
   // If we have blue in the lowest bit then obviously RGB 
   mode = ((window.ximage->blue_mask & 0x01) != 0) ? 1 : 2;
   /*
@@ -491,7 +490,7 @@ static void display_init_xshm()
     if (window.ximage->byte_order != LSBFirst) 
     #endif
     {
-    fprintf( stderr, "No support for non-native XImage byte order!\n" );
+    FATAL("No support for non-native XImage byte order!\n" );
     exit(1);
     }
   */
@@ -500,15 +499,164 @@ static void display_init_xshm()
 
 
 
-
-
-void display_init(yuv_image_t *picture_data,
-		  data_buf_head_t *picture_data_head,
-		  char *picture_buf_base)
+void init_config(Display *dpy)
 {
-  int screen;
-  Screen *scr;
+  int screen_nr;
+  char *dpy_str;
+  cfg_video_t *cfg_video = NULL;
+  cfg_display_t *cfg_display;
+  cfg_display_t *tmp_display;
+  DpyInfoOrigin_t orig;
   
+  screen_nr = DefaultScreen(dpy);
+  dpy_str = DisplayString(dpy);
+
+
+  
+  if(get_video_config(&cfg_video) == 0) {
+    ERROR("init_config(): Couldn't read any config files\n");
+  }
+  
+  DpyInfoInit(dpy, screen_nr);
+  
+  cfg_display = cfg_video->display;
+  tmp_display = cfg_display;
+
+  if(dpy_str != NULL) {
+    while(cfg_display) {
+      if(cfg_display->name != NULL) {
+	if(!strcmp(cfg_display->name, dpy_str)) {
+	  break;
+	}
+      } else {
+	tmp_display = cfg_display;
+      }
+      cfg_display = cfg_display->next;
+    }
+    
+    if(cfg_display == NULL) {
+      NOTE("using default config for '%s'\n", dpy_str);
+      cfg_display = tmp_display;
+    }
+  }
+  
+  DpyInfoSetUserGeometry(dpy, screen_nr,
+			 cfg_display->geometry.width,
+			 cfg_display->geometry.height);
+
+  DpyInfoSetUserResolution(dpy, screen_nr,
+			   cfg_display->resolution.horizontal_pixels,
+			   cfg_display->resolution.vertical_pixels);
+  
+  orig = DpyInfoOriginX11;
+  
+  if(cfg_display->geometry_src) {
+    if(!strcmp(cfg_display->geometry_src, "user")) {
+      orig = DpyInfoOriginUser;
+    } 
+  }
+  
+  orig = DpyInfoSetUpdateGeometry(dpy, screen_nr, orig);
+  {
+    char *orig_str;
+    switch(orig) {
+    case DpyInfoOriginX11:
+      orig_str = "X11";
+      break;
+    case DpyInfoOriginUser:
+      orig_str = "user";
+      break;
+    default:
+      orig_str = "";
+    }
+    NOTE("Using '%s' as source for geometry\n", orig_str);
+  }
+
+  orig = DpyInfoOriginX11;
+
+  if(cfg_display->resolution_src) {
+    if(!strcmp(cfg_display->resolution_src, "user")) {
+      orig = DpyInfoOriginUser;
+    } else if(!strcmp(cfg_display->resolution_src, "XF86VidMode")) {
+      orig = DpyInfoOriginXF86VidMode;
+    }
+  }
+  
+  orig = DpyInfoSetUpdateResolution(dpy, screen_nr, orig);
+  {
+    char *orig_str;
+    switch(orig) {
+    case DpyInfoOriginX11:
+      orig_str = "X11";
+      break;
+    case DpyInfoOriginXF86VidMode:
+      orig_str = "XF86VidMode";
+      break;
+    case DpyInfoOriginUser:
+      orig_str = "user";
+      break;
+    default:
+      orig_str = "";
+    }
+    NOTE("Using '%s' as source for resolution\n", orig_str);
+  }
+  
+  DpyInfoUpdateGeometry(dpy, screen_nr);
+  DpyInfoUpdateResolution(dpy, screen_nr);
+  
+  /* Query and calculate the displays aspect rate. */
+  {
+    int width, height;
+    int horizontal_pixels, vertical_pixels;
+    int dpy_sar_frac_n, dpy_sar_frac_d;
+
+    DpyInfoGetGeometry(dpy, screen_nr, &width, &height);
+    DpyInfoGetResolution(dpy, screen_nr, &horizontal_pixels, &vertical_pixels);
+    DpyInfoGetSAR(dpy, screen_nr, &dpy_sar_frac_n, &dpy_sar_frac_d);
+    
+    NOTE("Display w: %d, h: %d, hp: %d, vp: %d\n",
+	 width, height, horizontal_pixels, vertical_pixels);
+    NOTE("Display sar: %d/%d = %f\n",
+	 dpy_sar_frac_n, dpy_sar_frac_d,
+	 (double)dpy_sar_frac_n/(double)dpy_sar_frac_d);
+  }
+}
+
+void display_reset(void)
+{
+  //DNOTE("DEBUG[vo]: display_reset()\n");
+  if(use_xv) {
+    XShmDetach(mydisplay, &shm_info);
+    
+  } else if(use_xshm) {
+    
+    XShmDetach(mydisplay, &shm_info);
+    XDestroyImage(window.ximage);
+    if(shmdt(shm_info.shmaddr) == -1) {
+      FATAL("display_reset\n");
+      perror("shmdt");
+      exit(1);
+    }
+    
+    if(shmctl(shm_info.shmid, IPC_RMID, 0) == -1) {
+      FATAL("display_reset\n");
+      perror("shmctl");
+      exit(1);
+    }
+    
+    memset(&shm_info, 0, sizeof(XShmSegmentInfo));
+    
+  }
+
+}
+
+
+void display_init(int padded_width, int padded_height,
+		  int picture_buffer_shmid,
+		  char *picture_buffer_addr)
+{
+  static int initialized = 0;
+  Screen *scr;
   XSizeHints hint;
   XEvent xev;
   XGCValues xgcv;
@@ -517,145 +665,135 @@ void display_init(yuv_image_t *picture_data,
   XSetWindowAttributes xswa;
   unsigned long xswamask;
 
-  if(getenv("USE_FFB2_YUV2RGB")) {
-    use_ffb2_yuv2rgb = 1;
-  }
-  mydisplay = XOpenDisplay(NULL);
+  scale.image_width = padded_width;
+  scale.image_height = padded_height;
   
-  if(mydisplay == NULL) {
-    fprintf(stderr, "vo: Can not open display\n");
-    exit(1);
-  }
-  
-  screen = DefaultScreen(mydisplay);
-  scr = XDefaultScreenOfDisplay(mydisplay);
-  
-  
-  /* Query and calculate the displays aspect rate. */
-  dpy_size.width = DisplayWidthMM(mydisplay, screen);
-  dpy_size.height = DisplayHeightMM(mydisplay, screen);
-  
-  dpy_size.horizontal_pixels = DisplayWidth(mydisplay, screen);
-  dpy_size.vertical_pixels = DisplayHeight(mydisplay, screen);
-  
-  dpy_sar_frac_n = dpy_size.height * dpy_size.horizontal_pixels;
-  dpy_sar_frac_d = dpy_size.width * dpy_size.vertical_pixels;
-  
-  fprintf(stderr, "*d* h: %d, w: %d, hp: %d, wp: %d\n",
-	  dpy_size.height, dpy_size.width,
-	  dpy_size.vertical_pixels, dpy_size.horizontal_pixels);
-  fprintf(stderr, "*s* h: %d, w: %d, hp: %d, wp: %d\n",
-	  XHeightMMOfScreen(scr), XWidthMMOfScreen(scr),
-	  XHeightOfScreen(scr), XWidthOfScreen(scr));
-  fprintf(stderr, "*** display_sar: %d/%d\n", dpy_sar_frac_n, dpy_sar_frac_d);
-  
-  
-  /* Assume (for now) that the window will be the same size as the source. */
-  scale.image_width = picture_data->info->picture.horizontal_size;
-  scale.image_height = picture_data->info->picture.vertical_size;
-  
-  
-  /* Make the window */
-  XGetWindowAttributes(mydisplay, DefaultRootWindow(mydisplay), &attribs);
-  color_depth = attribs.depth;
-  if(color_depth != 15 && color_depth != 16 && 
-     color_depth != 24 && color_depth != 32) {
-    fprintf(stderr,"Only 15, 16, 24, and 32bpp supported. Trying 24bpp!\n");
-    color_depth = 24;
-  }
-  
-  XMatchVisualInfo(mydisplay, screen, color_depth, TrueColor, &vinfo);
-  fprintf(stderr, "vo: X11 visual id is %lx\n", vinfo.visualid);
-
-  hint.x = 0;
-  hint.y = 0;
-  hint.width = scale.image_width;
-  hint.height = scale.image_height;
-  hint.flags = PPosition | PSize;
-  
-  theCmap   = XCreateColormap(mydisplay, RootWindow(mydisplay,screen), 
-			      vinfo.visual, AllocNone);
-  
-  xswa.background_pixel = 0;
-  xswa.border_pixel     = 1;
-  xswa.colormap         = theCmap;
-  xswamask = CWBackPixel | CWBorderPixel | CWColormap;
-
-  window.win_state = WINDOW_STATE_NORMAL;
-
-  window.win = XCreateWindow(mydisplay, RootWindow(mydisplay,screen),
-			     hint.x, hint.y, hint.width, hint.height, 
-			     4, color_depth, CopyFromParent, vinfo.visual, 
-			     xswamask, &xswa);
-  
-  XSelectInput(mydisplay, window.win,
-	       StructureNotifyMask | ExposureMask |
-	       KeyPressMask | ButtonPressMask |
-	       PointerMotionMask | PropertyChangeMask);
-
-  create_transparent_cursor(mydisplay, window.win);
-  
-  /* Tell other applications/the window manager about us. */
-  snprintf(&title[0], 99, "Ogle v%s", VERSION);
-  XSetStandardProperties(mydisplay, window.win, &title[0], &title[0], 
-			 None, NULL, 0, &hint);
-  
-  /* Map window. */
-  XMapWindow(mydisplay, window.win);
-  
-  /* Wait for map. */
-  do {
-    XNextEvent(mydisplay, &xev);
-    switch(xev.type) {
-    case ConfigureNotify:
-      // remove all configure notify in queue
-      while(XCheckTypedEvent(mydisplay, ConfigureNotify, &xev) == True); 
-      
-      if(xev.xconfigure.window == window.win) {
-	Window dummy_win;
-	window.window_area.width = xev.xconfigure.width;
-	window.window_area.height = xev.xconfigure.height;
-
-	/*
-	display_adjust_size(current_image, 
-			    xev.xconfigure.width, 
-			    xev.xconfigure.height);
-	*/
-	window.video_area.width = scale.image_width;
-	window.video_area.height = scale.image_height;
-	window.video_area.x = (window.window_area.width - 
-			       window.video_area.width) / 2;
-	window.video_area.y = (window.window_area.height -
-			       window.video_area.height) / 2;
-
-	XTranslateCoordinates(mydisplay, window.win,
-			      DefaultRootWindow(mydisplay), 
-			      0,
-			      0,
-			      &window.window_area.x,
-			      &window.window_area.y,
-			      &dummy_win);
-      }
-      break;
-    default:
-      break;
+  if(!initialized) {
+    
+    if(getenv("USE_FFB2_YUV2RGB")) {
+      use_ffb2_yuv2rgb = 1;
     }
-  } while (xev.type != MapNotify || xev.xmap.event != window.win);
+    mydisplay = XOpenDisplay(NULL);
+    
+    if(mydisplay == NULL) {
+      FATAL("Cannot open display\n");
+      exit(1);
+    }
+    
+    screen_nr = DefaultScreen(mydisplay);
+    scr = XDefaultScreenOfDisplay(mydisplay);
+    
+    init_config(mydisplay);
+    
+    /* Assume (for now) that the window will be the same size as the source. */
+    scale.image_width = padded_width;
+    scale.image_height = padded_height;
+    
+    /* TODO search for correct visual ... */
+    /* Make the window */
+    XGetWindowAttributes(mydisplay, DefaultRootWindow(mydisplay), &attribs);
+    color_depth = attribs.depth;
+    if(color_depth != 15 && color_depth != 16 && 
+       color_depth != 24 && color_depth != 32) {
+      ERROR("Only 15, 16, 24, and 32bpp supported. Trying 24bpp!\n");
+      color_depth = 24;
+    }
+    
+    XMatchVisualInfo(mydisplay, screen_nr, color_depth, TrueColor, &vinfo);
+    //DNOTE("X11 visual id is %lx\n", vinfo.visualid);
+    
+    hint.x = 0;
+    hint.y = 0;
+    hint.width = scale.image_width;
+    hint.height = scale.image_height;
+    hint.flags = PPosition | PSize;
+    
+    theCmap   = XCreateColormap(mydisplay, RootWindow(mydisplay,screen_nr), 
+				vinfo.visual, AllocNone);
+    
+    xswa.background_pixel = 0;
+    xswa.border_pixel     = 1;
+    xswa.colormap         = theCmap;
+    xswamask = CWBackPixel | CWBorderPixel | CWColormap;
+    
+    window.win_state = WINDOW_STATE_NORMAL;
+    
+    window.win = XCreateWindow(mydisplay, RootWindow(mydisplay,screen_nr),
+			       hint.x, hint.y, hint.width, hint.height, 
+			       4, color_depth, CopyFromParent, vinfo.visual, 
+			       xswamask, &xswa);
+    
+    XSelectInput(mydisplay, window.win,
+		 StructureNotifyMask | ExposureMask |
+		 KeyPressMask | ButtonPressMask |
+		 PointerMotionMask | PropertyChangeMask);
+    
+    create_transparent_cursor(mydisplay, window.win);
+    
+    /* Tell other applications/the window manager about us. */
+    snprintf(&title[0], 99, "Ogle v%s", VERSION);
+    XSetStandardProperties(mydisplay, window.win, &title[0], &title[0], 
+			   None, NULL, 0, &hint);
+    
+    /* Map window. */
+    XMapWindow(mydisplay, window.win);
+    
+    /* Wait for map. */
+    do {
+      XNextEvent(mydisplay, &xev);
+      switch(xev.type) {
+      case ConfigureNotify:
+	// remove all configure notify in queue
+	while(XCheckTypedEvent(mydisplay, ConfigureNotify, &xev) == True); 
+	
+	if(xev.xconfigure.window == window.win) {
+	  Window dummy_win;
+	  window.window_area.width = xev.xconfigure.width;
+	  window.window_area.height = xev.xconfigure.height;
+	  
+	  /*
+	    display_adjust_size(current_image, 
+	    xev.xconfigure.width, 
+	    xev.xconfigure.height);
+	  */
+	  window.video_area.width = scale.image_width;
+	  window.video_area.height = scale.image_height;
+	  window.video_area.x = (window.window_area.width - 
+				 window.video_area.width) / 2;
+	  window.video_area.y = (window.window_area.height -
+				 window.video_area.height) / 2;
+	  
+	  XTranslateCoordinates(mydisplay, window.win,
+				DefaultRootWindow(mydisplay), 
+				0,
+				0,
+				&window.window_area.x,
+				&window.window_area.y,
+				&dummy_win);
+	}
+	break;
+      default:
+	break;
+      }
+    } while (xev.type != MapNotify || xev.xmap.event != window.win);
+    
+    XSync(mydisplay, False);
+    
+    /* Create the colormaps. (needed in the PutImage calls) */   
+    mygc = XCreateGC(mydisplay, window.win, 0L, &xgcv);
+    
+    
+    
+    
+    xshmeventbase = XShmGetEventBase(mydisplay);  
+    //DNOTE("xshmeventbase: %d\n", xshmeventbase);
+    
+  }
   
-  XSync(mydisplay, False);
-  
-  /* Create the colormaps. (needed in the PutImage calls) */   
-  mygc = XCreateGC(mydisplay, window.win, 0L, &xgcv);
-  
-
-  
-
-  xshmeventbase = XShmGetEventBase(mydisplay);  
-  fprintf(stderr, "xshmeventbase: %d\n", xshmeventbase);
-
   /* Try to use XFree86 Xv (X video) extension for display.
      Sets use_xv to true on success. */
-  display_init_xv(picture_data, picture_data_head, picture_buf_base);
+  display_init_xv(picture_buffer_shmid, picture_buffer_addr,
+		  padded_width, padded_height);
 
   /* Try XShm if we didn't have/couldn't use Xv. 
      This allso falls back to normal X11 if XShm fails. */
@@ -663,31 +801,33 @@ void display_init(yuv_image_t *picture_data,
     display_init_xshm();
   }
   
-  /*** sun ffb2 ***/
-  
-  if(use_ffb2_yuv2rgb) {  
-    fb_fd = open("/dev/fb", O_RDWR);
-    yuyv_fb = (unsigned int *)mmap(NULL, 4*1024*1024, PROT_READ | PROT_WRITE,
-				   MAP_SHARED, fb_fd, 0x0701a000);
-    if(yuyv_fb == MAP_FAILED) {
-      perror("mmap");
-      exit(-1);
-    }
+  if(!initialized) {
+    /*** sun ffb2+ ***/
     
-    rgb_fb = (uint8_t *)mmap(NULL, 4*2048*1024, PROT_READ | PROT_WRITE,
-				  MAP_SHARED, fb_fd, 0x05004000);
-    if(rgb_fb == MAP_FAILED) {
-      perror("mmap");
-      exit(-1);
+    if(use_ffb2_yuv2rgb) {  
+      fb_fd = open("/dev/fb", O_RDWR);
+      yuyv_fb = (unsigned int *)mmap(NULL, 4*1024*1024, PROT_READ | PROT_WRITE,
+				     MAP_SHARED, fb_fd, 0x0701a000);
+      if(yuyv_fb == MAP_FAILED) {
+	perror("mmap");
+	exit(1);
+      }
+      
+      rgb_fb = (uint8_t *)mmap(NULL, 4*2048*1024, PROT_READ | PROT_WRITE,
+			       MAP_SHARED, fb_fd, 0x05004000);
+      if(rgb_fb == MAP_FAILED) {
+	perror("mmap");
+	exit(1);
+      }
     }
+    /*** end sun ffb2+ ***/
+    
+    /* Let the user know what mode we are running in. */
+    snprintf(&title[0], 99, "Ogle v%s %s%s", VERSION, 
+	     use_xv ? "Xv " : "", use_xshm ? "XShm " : "");
+    XStoreName(mydisplay, window.win, &title[0]);
   }
-  /*** end sun ffb2 ***/
-
-  /* Let the user know what mode we are running in. */
-  snprintf(&title[0], 99, "Ogle v%s %s%s", VERSION, 
-	   use_xv ? "Xv " : "", use_xshm ? "XShm " : "");
-  XStoreName(mydisplay, window.win, &title[0]);
-  
+  initialized = 1;
 }
 
 
@@ -701,8 +841,8 @@ static void display_change_size(yuv_image_t *img, int new_width,
   int padded_height = img->info->picture.padded_height;
   //int alloc_width, alloc_height;
   int alloc_size;
-  fprintf(stderr, "vo resize: %d, %d\n", new_width, new_height);
   
+  DNOTE("resize: %d, %d\n", new_width, new_height);
   
   /* If we cant scale (i.e no Xv or mediaLib) exit give up now. */
 #ifndef HAVE_MLIB
@@ -729,13 +869,15 @@ static void display_change_size(yuv_image_t *img, int new_width,
     if(use_xshm)
       XShmDetach(mydisplay, &shm_info);
     XDestroyImage(window.ximage);
-    shmdt(shm_info.shmaddr);
-    if(shm_info.shmaddr == ((char *) -1)) {
-      fprintf(stderr, "vo: Shared memory: Couldn't detach segment\n");
+    
+    if(shmdt(shm_info.shmaddr) != 0) {
+      FATAL("display_change_size");
+      perror("shmdt");
       exit(1);
     }
-    if(shmctl(shm_info.shmid, IPC_RMID, 0) == -1) {
-      perror("vo: shmctl ipc_rmid");
+    if(shmctl(shm_info.shmid, IPC_RMID, 0) != 0) {
+      FATAL("display_change_size");
+      perror("shmctl");
       exit(1);
     }
     
@@ -751,7 +893,7 @@ static void display_change_size(yuv_image_t *img, int new_width,
 				    new_height);
 
     if(window.ximage == NULL) {
-      fprintf(stderr, "vo: Shared memory: couldn't create Shm image\n");
+      FATAL("XShmCreateImage failed\n");
       exit(1);
     }
 
@@ -764,20 +906,19 @@ static void display_change_size(yuv_image_t *img, int new_width,
     }
     
     /* Get a shared memory segment */
-    shm_info.shmid = shmget(IPC_PRIVATE,
-			    alloc_size,
-			    IPC_CREAT | 0777);
+    shm_info.shmid = shmget(IPC_PRIVATE, alloc_size, IPC_CREAT | 0777);
 
-
-    if(shm_info.shmid < 0) {
-      fprintf(stderr, "vo: Shared memory: Couldn't get segment\n");
+    if(shm_info.shmid == -1) {
+      FATAL("display_change_size");
+      perror("shmget");
       exit(1);
     }
-  
+    
     /* Attach shared memory segment */
     shm_info.shmaddr = (char *) shmat(shm_info.shmid, 0, 0);
     if(shm_info.shmaddr == ((char *) -1)) {
-      fprintf(stderr, "vo: Shared memory: Couldn't attach segment\n");
+      FATAL("display_change_size");
+      perror("shmat");
       exit(1);
     }
     
@@ -804,6 +945,7 @@ static void display_change_size(yuv_image_t *img, int new_width,
 
 static void display_adjust_size(yuv_image_t *current_image,
 				int given_width, int given_height) {
+  int dpy_sar_frac_n, dpy_sar_frac_d;
   int sar_frac_n, sar_frac_d; 
   int64_t scale_frac_n, scale_frac_d;
   int base_width, base_height, max_width, max_height;
@@ -820,15 +962,19 @@ static void display_adjust_size(yuv_image_t *current_image,
     sar_frac_n = current_image->info->picture.sar_frac_n;
     sar_frac_d = current_image->info->picture.sar_frac_d;
   }
-  
+
+  DpyInfoGetSAR(mydisplay, screen_nr, &dpy_sar_frac_n, &dpy_sar_frac_d);
+
   // TODO replace image->sar.. with image->dar
   scale_frac_n = (int64_t)dpy_sar_frac_n * (int64_t)sar_frac_d; 
   scale_frac_d = (int64_t)dpy_sar_frac_d * (int64_t)sar_frac_n;
   
-  fprintf(stderr, "vo: sar: %d/%d, dpy_sar %d/%d, scale: %lld, %lld\n",
-	  sar_frac_n, sar_frac_d,
-	  dpy_sar_frac_n, dpy_sar_frac_d,
-	  scale_frac_n, scale_frac_d); 
+#ifdef DEBUG
+  DNOTE("vo: sar: %d/%d, dpy_sar %d/%d, scale: %lld, %lld\n",
+	sar_frac_n, sar_frac_d,
+	dpy_sar_frac_n, dpy_sar_frac_d,
+	scale_frac_n, scale_frac_d); 
+#endif
   
   /* Keep either the height or the width constant. */ 
   if(scale_frac_n > scale_frac_d) {
@@ -840,7 +986,7 @@ static void display_adjust_size(yuv_image_t *current_image,
     base_height = (current_image->info->picture.vertical_size *
 		   scale_frac_d) / scale_frac_n;
   }
-  //fprintf(stderr, "vo: base %d x %d\n", base_width, base_height);
+  //DNOTE("base %d x %d\n", base_width, base_height);
   
   /* Do we have a predetermined size for the window? */ 
   if(given_width != -1 && given_height != -1 &&
@@ -850,14 +996,13 @@ static void display_adjust_size(yuv_image_t *current_image,
   } else {
     if(!scale.lock_window_size) {
       /* Never make the window bigger than the screen. */
-      max_width  = DisplayWidth(mydisplay, DefaultScreen(mydisplay));
-      max_height = DisplayHeight(mydisplay, DefaultScreen(mydisplay));
+      DpyInfoGetResolution(mydisplay, screen_nr, &max_width, &max_height);
     } else {
       max_width = window.window_area.width;
       max_height = window.window_area.height;
     }
   }
-  //fprintf(stderr, "vo: max %d x %d\n", max_width, max_height);
+  //DNOTE("max %d x %d\n", max_width, max_height);
   
   /* Fill the given area or keep the image at the same zoom level? */
   if((window.win_state == WINDOW_STATE_FULLSCREEN) 
@@ -867,12 +1012,12 @@ static void display_adjust_size(yuv_image_t *current_image,
     new_width  = max_width;
     new_height = (base_height * max_width) / base_width;
   } else {
-    fprintf(stderr, "vo: using zoom %d / %d\n", scale.zoom_n, scale.zoom_d);
+    //DNOTE("using zoom %d / %d\n", scale.zoom_n, scale.zoom_d);
     /* Use the provided zoom value. */
     new_width  = (base_width  * scale.zoom_n) / scale.zoom_d;
     new_height = (base_height * scale.zoom_n) / scale.zoom_d;
   }
-  //fprintf(stderr, "vo: new1 %d x %d\n", new_width, new_height);
+  //DNOTE("new1 %d x %d\n", new_width, new_height);
   
   /* Don't ever make it larger than the max limits. */
   if(new_width > max_width) {
@@ -883,7 +1028,7 @@ static void display_adjust_size(yuv_image_t *current_image,
     new_width  = (new_width * max_height) / new_height;
     new_height = max_height;
   }
-  //fprintf(stderr, "vo: new2 %d x %d\n", new_width, new_height);
+  //DNOTE("new2 %d x %d\n", new_width, new_height);
   
   /* Remeber what zoom level we ended up with. */
   if(window.win_state != WINDOW_STATE_FULLSCREEN) {
@@ -895,7 +1040,8 @@ static void display_adjust_size(yuv_image_t *current_image,
       scale.zoom_n = new_height;
       scale.zoom_d = base_height;
     }
-    fprintf(stderr, "vo: zoom2 %d / %d\n", scale.zoom_n, scale.zoom_d);
+    //DNOTE("zoom2 %d / %d\n", 
+    //        scale.zoom_n, scale.zoom_d);
   }
   
   /* Don't care about aspect and can't change the window size, use it all. */
@@ -917,9 +1063,10 @@ static void display_adjust_size(yuv_image_t *current_image,
 
 
 static void display_toggle_fullscreen(yuv_image_t *current_image) {
+
+  DpyInfoUpdateResolution(mydisplay, screen_nr);
   
   if(window.win_state != WINDOW_STATE_FULLSCREEN) {
-    
     ChangeWindowState(mydisplay, window.win, WINDOW_STATE_FULLSCREEN);
     window.win_state = WINDOW_STATE_FULLSCREEN;
   } else {
@@ -969,11 +1116,13 @@ void check_x_events(yuv_image_t *current_image)
 	  case EIDRM:
 #endif
 	  case EINVAL:
-	    fprintf(stderr, "vo: keypress: no msgq\n");
+	    FATAL("keypress\n");
+	    perror("MsgSendEvent");
 	    display_exit(); //TODO clean up and exit
 	    break;
 	  default:
-	    fprintf(stderr, "vo: keypress: couldn't send notification\n");
+	    FATAL("keypress, couldn't send notification\n");
+	    perror("MsgSendEvent");
 	    display_exit(); //TODO clean up and exit
 	    break;
 	  }
@@ -1014,19 +1163,19 @@ void check_x_events(yuv_image_t *current_image)
 	  case EIDRM:
 #endif
 	  case EINVAL:
-	    fprintf(stderr, "vo: buttonpress: no msgq\n");
+	    FATAL("buttonpress\n");
+	    perror("MsgSendEvent");
 	    display_exit(); //TODO clean up and exit
 	    break;
 	  default:
-	    fprintf(stderr, "vo: buttonpress: couldn't send notification\n");
+	    FATAL("buttonpress, couldn't send notification\n");
+	    perror("MsgSendEvent");
 	    display_exit(); //TODO clean up and exit
 	    break;
 	  }
 	}
-
       }
       
-
       if(cursor_visible == False) {
 	restore_cursor(mydisplay, window.win);
 	cursor_visible = True;
@@ -1062,11 +1211,13 @@ void check_x_events(yuv_image_t *current_image)
 	    case EIDRM:
 #endif
 	    case EINVAL:
-	      fprintf(stderr, "vo: pointermotion: no msgq\n");
+	      FATAL("pointermotion\n");
+	      perror("MsgSendEvent");
 	      display_exit(); //TODO clean up and exit
 	      break;
 	    default:
-	      fprintf(stderr, "vo: pointermotion: couldn't send notification\n");
+	      FATAL("pointermotion, couldn't send notification\n");
+	      perror("MsgSendEvent");
 	      display_exit(); //TODO clean up and exit
 	      break;
 	    }
@@ -1084,7 +1235,6 @@ void check_x_events(yuv_image_t *current_image)
       while(XCheckTypedEvent(mydisplay, Expose, &ev) == True);
       
       if(ev.xexpose.window == window.win) {
-
 	if(use_xv) {
 	  draw_win_xv(&window);
  	} else {
@@ -1119,7 +1269,6 @@ void check_x_events(yuv_image_t *current_image)
 			      &window.window_area.x,
 			      &window.window_area.y,
 			      &dummy_win);
-
 	
 	// top border
 	if(window.video_area.y > 0) {
@@ -1157,7 +1306,6 @@ void check_x_events(yuv_image_t *current_image)
 		     window.window_area.height,
 		     False);
 	}
-	
       }
       break;    
     default:
@@ -1242,7 +1390,6 @@ void display_exit(void)
       shmdt(shm_info.shmaddr);
     if(shm_info.shmid > 0) {
       shmctl(shm_info.shmid, IPC_RMID, 0);
-      fprintf(stderr, "vo: removed shm segment\n");
     }
   }
   display_process_exit();
@@ -1261,6 +1408,7 @@ static Bool predicate(Display *dpy, XEvent *ev, XPointer arg)
 
 static void draw_win_x11(window_info *dwin)
 {
+  int sar_frac_n, sar_frac_d; 
   char *address = dwin->ximage->data;
   
 #ifdef HAVE_MLIB
@@ -1280,8 +1428,25 @@ static void draw_win_x11(window_info *dwin)
     address += offs;
 #endif /* HAVE_MLIB */
 
-  /*** sun ffb2 ***/
 
+
+
+  if(screenshot || screenshot_spu) {
+    if(aspect_mode == AspectModeSrcVM) {
+      sar_frac_n // hack
+	= aspect_new_frac_d * dwin->image->info->picture.horizontal_size;
+      sar_frac_d // hack
+	= aspect_new_frac_n * dwin->image->info->picture.vertical_size;
+    }
+    /* Use the stream aspect */ 
+    else /* if(aspect_mode == AspectModeSrcMPEG) also default */ {
+      sar_frac_n = dwin->image->info->picture.sar_frac_n;
+      sar_frac_d = dwin->image->info->picture.sar_frac_d;
+    }
+  }
+
+
+  /*** sun ffb2 ***/
   if(use_ffb2_yuv2rgb) {
     int stride;
     int m;
@@ -1334,8 +1499,9 @@ static void draw_win_x11(window_info *dwin)
     
     clip_rect.x0 = 0;
     clip_rect.y0 = 0;
-    clip_rect.x1 = dpy_size.horizontal_pixels;
-    clip_rect.y1 = dpy_size.vertical_pixels;
+
+    DpyInfoGetResolution(mydisplay, screen_nr,
+			 &clip_rect.x1, &clip_rect.y1);
 
     fb_rect = clip(&fb_rect, &clip_rect);
     
@@ -1437,9 +1603,11 @@ static void draw_win_x11(window_info *dwin)
   
   if(screenshot_spu) {
     screenshot_spu = 0;
+
     screenshot_rgb_jpg(address,
 		       dwin->image->info->picture.padded_width,
-		       dwin->image->info->picture.padded_height);
+		       dwin->image->info->picture.padded_height,
+		       sar_frac_n, sar_frac_d);
   }
   
 
@@ -1476,7 +1644,8 @@ static void draw_win_x11(window_info *dwin)
   
   if(screenshot) {
     screenshot = 0;
-    screenshot_yuv_jpg(dwin->image, dwin->ximage);
+    
+    screenshot_yuv_jpg(dwin->image, dwin->ximage, sar_frac_n, sar_frac_d);
   }
   
   window.video_area.width = scale.image_width;
@@ -1500,7 +1669,13 @@ static void draw_win_x11(window_info *dwin)
 	  window.video_area.height,
 	  window.video_area.height);
   */
-
+  /*
+    DNOTE("window.video_area. x:%d, y: %d, w:%d, h:%d\n",
+	  window.video_area.x,
+	  window.video_area.y,
+	  window.video_area.width,
+	  window.video_area.height);
+  */
   if(use_xshm) {
     XShmPutImage(mydisplay, dwin->win, mygc, dwin->ximage, 0, 0, 
 		 window.video_area.x,
@@ -1508,7 +1683,6 @@ static void draw_win_x11(window_info *dwin)
 		 window.video_area.width,
 		 window.video_area.height,
 		 True);
-
     
     {
       XEvent ev;
@@ -1525,38 +1699,50 @@ static void draw_win_x11(window_info *dwin)
 	      window.video_area.height);
     XSync(mydisplay, False);
   }
-  
 }
 
 
 static void draw_win_xv(window_info *dwin)
 {
 #ifdef HAVE_XV
-  unsigned char *dst;
-  
-  dst = xv_image->data;
-  
+  int sar_frac_n, sar_frac_d;   
   /* Set the source of the xv_image to the source of the image 
      that we want drawn. */ 
   xv_image->data = dwin->image->y;
 
+  if(screenshot || screenshot_spu) { 
+    if(aspect_mode == AspectModeSrcVM) {
+      sar_frac_n // hack
+	= aspect_new_frac_d * dwin->image->info->picture.horizontal_size;
+      sar_frac_d // hack
+	= aspect_new_frac_n * dwin->image->info->picture.vertical_size;
+    }
+    /* Use the stream aspect */ 
+    else /* if(aspect_mode == AspectModeSrcMPEG) also default */ {
+      sar_frac_n = dwin->image->info->picture.sar_frac_n;
+      sar_frac_d = dwin->image->info->picture.sar_frac_d;
+    }
+  }
+  
   if(screenshot) {
     screenshot = 0;
-    screenshot_yuv_jpg(dwin->image, dwin->ximage);
+    
+    screenshot_yuv_jpg(dwin->image, dwin->ximage, sar_frac_n, sar_frac_d);
   }
     
 #ifdef SPU
   if(msgqid != -1) {
     //ugly hack
-    if(mix_subpicture_yuv(dwin->image, reserv_image)) {
-      xv_image->data = reserv_image->y;
+    if(mix_subpicture_yuv(dwin->image, cur_data_q->reserv_image)) {
+      xv_image->data = cur_data_q->reserv_image->y;
     }
   }
 #endif
 
   if(screenshot_spu) {
     screenshot_spu = 0;
-    screenshot_yuv_jpg(dwin->image, dwin->ximage);
+    
+    screenshot_yuv_jpg(dwin->image, dwin->ximage, sar_frac_n, sar_frac_d);
   }
   
   window.video_area.width = scale.image_width;
@@ -1589,7 +1775,3 @@ static void draw_win_xv(window_info *dwin)
 #endif /* HAVE_XV */
 }
 
-
-
-
-
