@@ -99,7 +99,12 @@ typedef enum {
   MsgEventQInputButtonRelease,
   MsgEventQInputPointerMotion,
   MsgEventQInputKeyPress,
-  MsgEventQInputKeyRelease
+  MsgEventQInputKeyRelease,
+  MsgEventQDestroyBuf,
+  MsgEventQAppendQ,
+  MsgEventQDetachQ,
+  MsgEventQQDetached,
+  MsgEventQDestroyQ
 } MsgEventType_t;
 
 
@@ -122,6 +127,7 @@ typedef struct {
   int y_root;
   unsigned long mod_mask; /* X modifiers (keys/buttons)*/
   unsigned long input;
+  unsigned long time; /* milliseconds */
 } MsgQInputEvent_t;
 
 
@@ -220,6 +226,13 @@ typedef struct {
   int size;
 } MsgQReqBufEvent_t;
 
+typedef struct {
+  MsgEventType_t type;
+  MsgEventQ_t *q;
+  MsgEventClient_t client;
+  int shmid;
+} MsgQDestroyBufEvent_t;
+
 typedef struct {
   MsgEventType_t type;
   MsgEventQ_t *q;
@@ -318,6 +331,13 @@ typedef struct {
   int q_shmid;
 } MsgQAttachQEvent_t;
 
+typedef struct {
+  MsgEventType_t type;
+  MsgEventQ_t *q;
+  MsgEventClient_t client;
+  int q_shmid;
+} MsgQDetachQEvent_t;
+
 typedef struct {
   MsgEventType_t type;
   MsgEventQ_t *q;
@@ -332,7 +352,7 @@ typedef struct {
   MsgEventType_t type;
   MsgEventQ_t *q;
   MsgEventClient_t client;
-  int to_scrnr;
+  int to_scrid;
 } MsgQFlushDataEvent_t;
 
 typedef struct {
@@ -414,10 +434,12 @@ typedef union {
   MsgQDecodeStreamBufEvent_t decodestreambuf;
   MsgQReqBufEvent_t reqbuf;
   MsgQGntBufEvent_t gntbuf;
+  MsgQDestroyBufEvent_t destroybuf;
   MsgQCtrlDataEvent_t ctrldata;
   MsgQReqPicBufEvent_t reqpicbuf;
   MsgQGntPicBufEvent_t gntpicbuf;
   MsgQAttachQEvent_t attachq;
+  MsgQDetachQEvent_t detachq;
   MsgQSPUPaletteEvent_t spupalette;
   MsgQSPUHighlightEvent_t spuhighlight;
   MsgQSpeedEvent_t speed;
